# -*- coding: utf-8 -*-
from unittest import TestCase
from underthesea.transformer.unicode import UnicodeTransformer


class TestUnicodeTransformer(TestCase):
    def test_transform_1(self):
        transformer = UnicodeTransformer()
        s = "tôi đi học"
        self.assertEqual(unicode, type(transformer.transform(s)))

    def test_transform_2(self):
        transformer = UnicodeTransformer()
        s = u"tôi đi học"
        self.assertEqual(unicode, type(transformer.transform(s)))




# -*- coding: utf-8 -*-
from unittest import TestCase
from os import listdir
from os.path import dirname, join
import io
from tests.word_sent.test_config import EXPECTED_SPEED
from underthesea import word_sent
import time
from underthesea.word_sent.tokenize import tokenize


class TestPerformance(TestCase):
    def setUp(self):
        input_folder = join(dirname(__file__), "samples", "4_documents")
        files = listdir(input_folder)
        files = [join(input_folder, file) for file in files]
        texts = [io.open(file, "r", encoding="utf-8").read().split("\n") for file in files]
        texts = [text for sublist in texts for text in sublist]
        self.texts = texts

    def test_1(self):
        n_tokens = 0
        for text in self.texts:
            n_tokens += len(tokenize(text).split(" "))
        start = time.time()
        for text in self.texts:
            word_sent(text)
        end = time.time()
        duration = end - start  # in seconds
        speed = n_tokens / duration
        print "Speed: ", speed
        self.assertGreater(speed, EXPECTED_SPEED)




from underthesea import word_sent
from underthesea.pos_tag.model import CRFModel


def pos_tag(sentence, format=None):
    """
    part of speech tagging
    
    :param unicode|str sentence: raw sentence
    :return: tagged sentence 
    :rtype: list 
    """
    sentence = word_sent(sentence)
    crf_model = CRFModel.Instance()
    result = crf_model.predict(sentence, format)
    return result




from underthesea.word_sent.feature import word2features


def sent2features(sent):
    """

    :param unicode|str sent: sentence has been converted to column format
    :return: words of sentence added feature
    :rtype: list
    """
    return [word2features(sent, i) for i in range(len(sent))]


class Transformer:
    def __init__(self):
        pass

    @staticmethod
    def transform(sentence):
        """
        :param unicode|str sentence: raw sentence
        :return: words of sentence added feature
        :rtype: list
        """
        sentence = [(token,) for token in sentence.split()]
        return sent2features(sentence)




# -*- coding: utf-8 -*-
from os import listdir
from unittest import TestCase
from underthesea import pos_tag
from os.path import dirname, join

samples_dir = join(dirname(__file__), "samples")


def load_input(input_file):
    content = [text.split("\t")[0].decode("utf-8") for text in open(input_file, "r").read().strip().split("\n")]
    content = u" ".join(content)
    return content


def load_output(input_file):
    lines = [text.split("\t") for text in open(input_file, "r").read().strip().split("\n")]
    output = []
    for item in lines:
        word, tag = item
        output.append((word.decode("utf-8"), tag))
    return output


def save_temp(id, output):
    temp_file = join(samples_dir, "%s.correct" % id)
    content = u"\n".join([u"\t".join(item) for item in output])
    open(temp_file, "w").write(content.encode("utf-8"))


class TestPosTag(TestCase):
    def test_simple_cases(self):
        sentence = u""
        actual = pos_tag(sentence)
        expected = []
        self.assertEqual(actual, expected)

    def test_accuracy(self):
        test_dir = join(dirname(__file__), "samples")
        files = listdir(test_dir)
        ids = [f.split(".")[0] for f in files]
        for id in ids:
            file = join(test_dir, "%s.txt" % id)
            sentence = load_input(file)
            actual = pos_tag(sentence)
            expected = load_output(file)
            if actual != expected:
                print("Fail {}".format(id))
                save_temp(id, actual)
            self.assertEqual(actual, expected)



# -*- coding: utf-8 -*-
from unittest import TestCase
from os.path import dirname, join
import io
import time
from tests.word_sent.test_config import EXPECTED_SPEED
from underthesea.word_sent.tokenize import tokenize
from underthesea.word_sent.word_sent import word_sent


class TestPerformance2(TestCase):
    def setUp(self):
        file = join(dirname(__file__), "samples", "sentences", "448_tokens.txt")
        text = io.open(file, "r", encoding="utf-8").read()
        self.text = text

    def test_1(self):
        n_tokens = len(tokenize(self.text).split(" "))
        start = time.time()
        word_sent(self.text)
        end = time.time()
        duration = end - start  # in seconds
        if duration != 0:
            speed = n_tokens / duration
            print "Speed: ", speed
            self.assertGreater(speed, EXPECTED_SPEED)



from os.path import dirname, join
import pycrfsuite
from underthesea.util.singleton import Singleton
from underthesea.word_sent.transformer import Transformer


@Singleton
class CRFModel:
    def __init__(self):
        """
        load trained data from crf-model
        """
        self.model = pycrfsuite.Tagger()
        filepath = join(dirname(__file__), "crf-model")
        self.model.open(filepath)
        filepath = join(dirname(__file__), "punctuation.txt")
        self.punctuation = open(filepath, "r").read().split("\n")

    def predict(self, sentence, format=None):
        """

        :param unicode|str sentence: raw sentence
        :return: segmented sentence
        :rtype: unicode|str
        """
        tokens = Transformer.transform(sentence)
        tags = self.model.tag(tokens)
        if len(tags) > 0:
            tags[0] = "B_W"
        tokens = [item[0] for item in tokens]
        output = []
        for tag, token in zip(tags, tokens):
            if tag == "I_W":
                output[-1] = output[-1] + u" " + token
            else:
                output.append(token)
        if format == "text":
            output = u" ".join([item.replace(" ", "_") for item in output])
        return output




from os.path import dirname
from os.path import join

from underthesea.pos_tag.feature import word2features


def sent2features(sent, template):
    return [word2features(sent, i, template) for i in range(len(sent))]


class Transformer:
    def __init__(self):
        pass

    @staticmethod
    def transform(sentence):
        template = [
            "T[0].lower", "T[-1].lower", "T[1].lower",
            "T[0].istitle", "T[-1].istitle", "T[1].istitle",
            "T[-2]", "T[-1]", "T[0]", "T[1]", "T[2]",  # unigram
            "T[-2,-1]", "T[-1,0]", "T[0,1]", "T[1,2]",  # bigram
            "T[-1][1]", "T[-2][1]", "T[-3][1]",  # dynamic feature
            "T[-3,-2][1]", "T[-2,-1][1]",
            "T[-3,-1][1]"
        ]
        sentence = [(token, "A") for token in sentence]
        return sent2features(sentence, template)

    @staticmethod
    def extract_features(sentence, template):
        return sent2features(sentence, template)

    def format_word(self, sentence):
        path = join(dirname(dirname(dirname(__file__))), "pipelines", "logs", "punctuation.txt")
        punctuations = open(path, "r").read().split("\n")
        for punctuation in punctuations:
            punctuation = unicode(punctuations)
        words = []
        for word in sentence.split(" "):
            if "_" in word:
                tokens = []
                word = word.replace("_", " ")
                for token in word.split(" "):
                    if token != "":
                        tokens.append(token)

                for i in range(tokens.__len__()):
                    if i != 0:
                        tokens[i] += "\tI_W"
                    else:
                        tokens[i] += "\tB_W"
                    words.append(tokens[i])
            elif word in punctuations:
                words.append(word + "\tO")
            else:
                words.append(word + "\tB_W")
        return words

    def list_to_tuple(self, sentences):
        word_tuple = []
        for i in sentences:
            arr = i.split('\t')
            word_tuple.append((arr[0], arr[1]))
        return word_tuple


def sent2labels(sent):
    return [label for token, label in sent]



from underthesea.corpus.corpus import *
from underthesea.corpus.plaintext import *
from underthesea.corpus.readers.dictionary_loader import DictionaryLoader

viet_dict_11K = DictionaryLoader('Viet11K.txt')
viet_dict_22K = DictionaryLoader('Viet22K.txt')
viet_dict_39K = DictionaryLoader('Viet39K.txt')
viet_dict_74K = DictionaryLoader('Viet74K.txt')



import re
from os.path import dirname, join
import pycrfsuite
from transformer import Transformer
from underthesea.util.singleton import Singleton


@Singleton
class CRFModel:
    def __init__(self):
        self.model = pycrfsuite.Tagger()
        filepath = join(dirname(__file__), "postag_crf_v1.model")
        self.model.open(filepath)

    def predict(self, sentence, format=None):
        x = Transformer.transform(sentence)
        tags = self.model.tag(x)
        tokenized_sentence = u''
        return zip(sentence, tags)



# ===========================
# token syntax
# ===========================
#         _ row 1
#        /  _ row 2
#       /  /  _ column
#      /  /  /
#    T[0,2][0]
#          .is_digit
#            \_ function
#
# ===========================
# sample tagged sentence
# ===========================
# this     A
# is       B
# a        C
# sample   D
# sentence E
#

import re


def text_lower(word):
    return word.lower()


def text_istitle(word):
    if len(word) == 0:
        return False
    if type(word) == str:
        word = word.decode("utf-8")
    try:
        titles = [s[0] for s in word.split(" ")]
        for token in titles:
            if token[0].istitle() is False:
                return False
        return True
    except:
        return False


def apply_function(name, word):
    functions = {
        "lower": text_lower,
        "istitle": text_istitle
    }
    return functions[name](word)


def template2features(sent, i, token, debug=True):
    """
    :type token: object
    """
    columns = [
        [t[0] for t in sent],
        [t[1] for t in sent]
    ]
    matched = re.match("T\[(?P<index1>\-?\d+)(\,(?P<index2>\-?\d+))?\](\[(?P<column>.*)\])?(\.(?P<function>.*))?", token)
    column = matched.group("column")
    column = int(column) if column else 0
    index1 = int(matched.group("index1"))
    index2 = matched.group("index2")
    index2 = int(index2) if index2 else None
    func = matched.group("function")
    if i + index1 < 0:
        return ["%s=BOS" % token]
    if i + index1 >= len(sent):
        return ["%s=EOS" % token]
    if index2 is not None:
        if i + index2 >= len(sent):
            return ["%s=EOS" % token]
        word = " ".join(columns[column][i + index1: i + index2 + 1])
    else:
        word = sent[i + index1][column]
    if func is not None:
        result = apply_function(func, word)
    else:
        result = word
    return ["%s=%s" % (token, result)]


def word2features(sent, i, template):
    features = []
    for token in template:
        features.extend(template2features(sent, i, token))
    return features



import os
import shutil
from os import listdir
from unittest import TestCase
from os.path import join
from underthesea.corpus import PlainTextCorpus

test_folder = os.path.dirname(__file__)


class TestPlainTextCorpus(TestCase):
    def setUp(self):
        self.plaintext_folder = join(test_folder, "sample_text_corpus")
        self.saved_plaintext_folder = join(test_folder, "sample_saved_text_corpus")

    def test___init__(self):
        corpus = PlainTextCorpus()
        self.assertIsNone(corpus.documents)

    def test_load(self):
        corpus = PlainTextCorpus()
        corpus.load(self.plaintext_folder)
        self.assertEqual(4, len(corpus.documents))

    def test_sentences(self):
        corpus = PlainTextCorpus()
        corpus.load(self.plaintext_folder)
        self.assertEqual(list, type(corpus.documents[0].sentences))

    def test_unicode(self):
        corpus = PlainTextCorpus()
        corpus.load(self.plaintext_folder)
        self.assertEqual(unicode, type(corpus.documents[0].sentences[0]))

    def test_save(self):
        corpus = PlainTextCorpus()
        corpus.load(self.plaintext_folder)
        corpus.save(self.saved_plaintext_folder)
        files = listdir(self.saved_plaintext_folder)
        self.assertEqual(4, len(files))
        try:
            shutil.rmtree(self.saved_plaintext_folder)
        except:
            pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

requirements = [
    'Click>=6.0',
    'python-crfsuite==0.9.1'
]

test_requirements = [
    'nose==1.3.7'
]

setup(
    name='underthesea',
    version='1.0.19',
    description="Vietnamese NLP Toolkit",
    long_description=readme + '\n\n' + history,
    author="Vu Anh",
    author_email='brother.rain.1024@gmail.com',
    url='https://github.com/magizbox/underthesea',
    packages=[
        'underthesea',
    ],
    package_dir={'underthesea': 'underthesea'},
    entry_points={
        'console_scripts': [
            'underthesea=underthesea.cli:main'
        ]
    },
    include_package_data=True,
    install_requires=requirements,
    license="GNU General Public License v3",
    zip_safe=False,
    keywords='underthesea',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Natural Language :: English',
        'Programming Language :: Python :: 2.7'
    ],
    test_suite='tests',
    tests_require=test_requirements
)



# -*- coding: utf-8 -*-
from os import listdir
from unittest import TestCase
from underthesea import word_sent
from os.path import dirname, join


def load_input(input_file):
    return open(input_file, "r").read().split("\n")[0].decode("utf-8")


def load_output(output_file):
    return [text.decode("utf-8") for text in open(output_file, "r").read().strip().split("\n")]


def save_temp(id, output):
    test_dir = join(dirname(__file__), "samples", "accuracy")
    temp_file = join(test_dir, "%s.tmp" % id)
    content = u"\n".join(output)
    open(temp_file, "w").write(content.encode("utf-8"))


class TestWord_sent(TestCase):
    def test_simple_cases(self):
        sentence = u""
        actual = word_sent(sentence)
        expected = []
        self.assertEqual(actual, expected)

        actual = word_sent(sentence, format="text")
        expected = u""
        self.assertEqual(actual, expected)

    def test_word_sent(self):
        test_dir = join(dirname(__file__), "samples", "accuracy")
        files = listdir(test_dir)
        ids = [f.split(".")[0] for f in files if ".in" in f]
        for id in ids:
            input_file = join(test_dir, "%s.in" % id)
            output_file = join(test_dir, "%s.out" % id)
            sentence = load_input(input_file)
            actual = word_sent(sentence)
            expected = load_output(output_file)
            if actual != expected:
                print("Fail {}".format(id))
                save_temp(id, actual)
            self.assertEqual(actual, expected)
